-- ReSeq v0.6 - By Aftab Hussain aka afta8 | fathand@gmail.com - 15/10/13 

--[[============================================================================
main.lua
============================================================================]]--

-- Placeholder for the dialog
local dialog = nil

-- Placeholder to expose the ViewBuilder outside the show_dialog() function
local vb = nil

-- Reload the script whenever this file is saved. 
-- Additionally, execute the attached function.
_AUTO_RELOAD_DEBUG = function()
  
end

-- Read from the manifest.xml file.
class "RenoiseScriptingTool" (renoise.Document.DocumentNode)
  function RenoiseScriptingTool:__init()    
    renoise.Document.DocumentNode.__init(self) 
    self:add_property("Name", "Untitled Tool")
    self:add_property("Id", "Unknown Id")
    self:add_property("Version", "Unknown Version") -- Added for use in help box
  end

local manifest = RenoiseScriptingTool()
local ok,err = manifest:load_from("manifest.xml")
local tool_name = manifest:property("Name").value
local tool_id = manifest:property("Id").value
local tool_version = manifest:property("Version").value -- Added for use in help box

--------------------------------------------------------------------------------
-- Main functions
--------------------------------------------------------------------------------

-- Set up preferences
local options = renoise.Document.create("ScriptingToolPreferences") {
  launch_count = 0,
  cutoff_curve = 1
}
renoise.tool().preferences = options

-- Variables 
local ins 
local p_env 
local v_env 
local pn_env 
local c_env 
local r_env


-- Envelope length
local env_length = 97
-- Number of envelope ticks per step 
local env_tpl = 24/4 


-- Rounding to a step value
function round_step(n, step)
  return math.floor((n/step) + 0.5)*step
end

-- Clamp values
local function clamp_value(input, min_val, max_val)
  return math.min(math.max(input, min_val), max_val)
end

-- Return the sign of a value
local function sign(x)
  return (x<0 and -1) or 1
end

-- Apply a curve to a linear value
function curve(x,s, linear)
  if linear ~= true then
    return math.exp(math.log(16)*x^s)/15-1/15 -- Return the curved value
  else
    return ((math.log((x + 1/15)*15))/math.log(16))^1/s -- Return the linear value from the curved value
  end
end


-- Initialise envelope settings
local function initialise_envelopes()
  -- Initialise pitch envelope
  p_env:clear_points() 
  p_env.length = env_length
  p_env.play_mode = renoise.InstrumentEnvelope.PLAYMODE_POINTS 
  p_env.loop_mode = renoise.InstrumentEnvelope.LOOP_MODE_FORWARD
  p_env.loop_start = 1 
  p_env.loop_end = env_length
  p_env.enabled = true
  p_env.sustain_enabled = false

  -- Initialise volume envelope
  v_env:clear_points() 
  v_env.length = env_length
  v_env.play_mode = renoise.InstrumentEnvelope.PLAYMODE_POINTS 
  v_env.loop_mode = renoise.InstrumentEnvelope.LOOP_MODE_FORWARD
  v_env.loop_start = 1 
  v_env.loop_end = env_length
  v_env.enabled = true
  v_env.sustain_enabled = false
  v_env.fade_amount = 4095
  
  -- Initialise cutoff envelope
  c_env:clear_points() 
  c_env.length = env_length
  c_env.play_mode = renoise.InstrumentEnvelope.PLAYMODE_POINTS 
  c_env.loop_mode = renoise.InstrumentEnvelope.LOOP_MODE_FORWARD
  c_env.loop_start = 1 
  c_env.loop_end = env_length
  c_env.enabled = true
  c_env.sustain_enabled = false

  -- Initialise resonance envelope
  r_env:clear_points() 
  r_env.length = env_length
  r_env.play_mode = renoise.InstrumentEnvelope.PLAYMODE_POINTS 
  r_env.loop_mode = renoise.InstrumentEnvelope.LOOP_MODE_FORWARD
  r_env.loop_start = 1 
  r_env.loop_end = env_length
  r_env.enabled = true
  r_env.sustain_enabled = false
  
end


-- Return an envelope time position for a given step number
local function env_tick(step)
  local line_ticks = ((step-1)*env_tpl)+1
  return line_ticks
end


-- Insert pitch point 
local function insert_pitch(step, note)   
  note = (note-1)/24 -- Convert note pitch to envelope value
  p_env:add_point_at(env_tick(step), note) -- Set the pitch envelop point
end


-- Refresh envelope note off points
local function refresh_note_offs(step_data, step, max_value)
  -- Get the note length in ticks
  local length_ticks = (step_data[step].length/max_value)*env_tpl -- Scale it from the GUI length (max_value) 
  length_ticks = round_step(length_ticks, 1) -- Round to integer
  local step_tick = env_tick(step) -- Get the tick value for the step  
  -- Delete any existing note off points
  for tick = 1, 5 do
    if v_env:has_point_at(step_tick+tick) then
      v_env:remove_point_at(step_tick+tick) 
    end
  end
  -- Add the new note off point
  if length_ticks ~= env_tpl then -- To avoid overwriting the next note-on
    v_env:add_point_at(step_tick+length_ticks, 0)
  end    
end


-- Refresh pitch & volume envelope points
local function refresh_points(step_data, step)
  if step_data[step].note == 0 then 
    v_env:add_point_at(env_tick(step), 0)
    if p_env:has_point_at(env_tick(step)) then 
      p_env:remove_point_at(env_tick(step))
    end
  else
    insert_pitch(step, step_data[step].note)
    v_env:add_point_at(env_tick(step), step_data[step].volume)  
  end
end


-- Refresh cutoff and resonance envelope points
local function refresh_cutoff(step_data, step)
  c_env:add_point_at(env_tick(step), step_data[step].cutoff)  
end

local function refresh_resonance(step_data, step) 
  r_env:add_point_at(env_tick(step), step_data[step].resonance)
end


--------------------------------------------------------------------------------
-- GUI
--------------------------------------------------------------------------------

local function show_dialog()

  -- This block makes sure a non-modal dialog is shown once.
  -- If the dialog is already opened, it will be focused.
  if dialog and dialog.visible then
    dialog:show()
    return
  end
  
  -- The ViewBuilder is the basis
  vb = renoise.ViewBuilder()

  -- Increment launch count
  options.launch_count.value = options.launch_count.value + 1
  
  -- Curve shape for cutoff controls 1 & 2 for exp, 0.1 & 0.2 for log
  local curve_shape = options.cutoff_curve.value

  -- Select current instrument
  ins = renoise.song().selected_instrument_index
  p_env = renoise.song().instruments[ins].sample_envelopes.pitch
  v_env = renoise.song().instruments[ins].sample_envelopes.volume
  pn_env = renoise.song().instruments[ins].sample_envelopes.pan
  c_env = renoise.song().instruments[ins].sample_envelopes.cutoff
  r_env = renoise.song().instruments[ins].sample_envelopes.resonance

  -- Sequencer GUI settings
  local num_steps = 16
  local num_notes = 25
  local spacing = -2

  -- Slider scale variables used to remember the last slider setting
  local prev_gate_val = 0
  local prev_volume_val = 0
  local prev_cutoff_val = 0
  local prev_resonance_val = 0
  local scale_sens = 0.0075 -- Sensitivity of the scaling effect

  -- Button settings
  local max = 32
  local min = max/env_tpl
  local b_height = 12
  local on_colour = {200,0,0}   
  local function off_colour(note,step) -- Returns the off colour based on the note number
    local keys = {0,1,0,1,0,0,1,0,1,0,1,0,0,1,0,1,0,0,1,0,1,0,1,0,0}
    local white_key = {217,217,217}
    local black_key = {161,161,161}
    local beat_white_key = {200,200,217}
    if keys[note] == 0 then
      if (step == 1) or (step == 5) or (step == 9) or (step == 13) then
        return beat_white_key
      else
        return white_key
      end
    elseif keys[note] == 1 then
      return black_key
    end
  end


  -- Create table for sequencer data and initialise
  local step_data = {}
  
  -- Populate step_data with initial settings
  for step = 1, num_steps do
    step_data[step] = { note = 0, length = max, volume = 1, cutoff = 1, resonance = 0 }
  end

  
  -- Update button length GUI
  local function update_button_length(step, note, length)
    if note ~= 0 then
      vb.views["button_light_step"..step.."note"..note].width =  round_step(length, min)
    end
  end

  
  -- Function to update button state GUI
  local function update_button_state(step, note, status)
    if status == 0 then -- The current note has been turned off
      vb.views["button_light_step"..step.."note"..note].color = off_colour(note, step)
      update_button_length(step, note, max) -- Reset the button display length
    else -- A new note is being turned on
      if step_data[step].note ~= 0 then -- Turn off the note that is already on
        vb.views["button_light_step"..step.."note"..step_data[step].note].color = off_colour(step_data[step].note, step)
        update_button_length(step, step_data[step].note, max) -- Reset the button display length  
      end
      -- Now turn on the selected note passed to the function
      vb.views["button_light_step"..step.."note"..note].color = on_colour
      update_button_length(step, note, step_data[step].length) -- Set the button length    
    end  
  end
    
    
  -- Create Sequencer Grid
  local sequencer_row = vb:row {spacing = spacing}
  for step = 1, num_steps do  
    local button_column = vb:column {spacing = spacing}

    
    -- Create text for envelope command
    local env_commands = {"00","0F","1F","2F","3F","4F","5F","6F","7F","8F","9F","AF","BF","CF","DF","EF"}
    local env_command_text = vb:row {
      vb:text {
        width = max,
        text = env_commands[step],
        align = "center",
        font = "mono"
      }
    }    
    button_column:add_child(env_command_text) 

    
    -- Create piano roll
    for note = num_notes, 1, -1 do
      local note_button = vb:row {
        spacing = -1*max, -- Negative spacing enables overlapping buttons
        -- Create background note button
        vb:button {
          id = "button_dark_step"..step.."note"..note,
          width = max,
          height = b_height,
          color = off_colour(note, step),
          pressed = function() -- Turn pressed note off (The dark button is only clickable if the note is already on)
            update_button_state(step, note, 0) 
            step_data[step].note = 0
            refresh_points(step_data, step)
          end
        },
        -- Create visible note button on top of background note button    
        vb:button {
          id = "button_light_step"..step.."note"..note,
          width = max,
          height = b_height,
          color = off_colour(note, step),
          pressed = function()
            if step_data[step].note == note then -- If the note is already on then turn it off
              update_button_state(step, note, 0) 
              step_data[step].note = 0
              refresh_points(step_data, step)                         
            else -- Otherwise turn the note on
              update_button_state(step, note, 1)
              if step_data[step].volume == 0 then -- Set the volume to be full if it is zero
                step_data[step].volume = 1 
              end
              step_data[step].note = note 
              refresh_points(step_data, step)              
            end
          end
        }
      }            
      button_column:add_child(note_button)      
    end
    

    -- Create Sequencer Controls
    local step_controls = vb:column {
      style = "group",

      -- Note Length slider      
      vb:row {
        margin = -3,
        vb:minislider {
          id = "length_step"..step,
          --tooltip = "Note length",
          min = min,
          max = max,
          value = max,
          width = max+6,
          height = 20,
          notifier = function(value)          
            -- Update the length setting for the particular step
            step_data[step].length = value
            -- Update the note button length display
            update_button_length(step, step_data[step].note, value) 
            -- Update the note off envelope points
            refresh_note_offs(step_data, step, max)       
          end
        } 
      },
 
      vb:space { height = 3 },
 
      -- Volume dial
      vb:rotary {
        id = "volume_step"..step,
        --tooltip = "Volume",
        min = 0,
        max = 1,
        value = 1,
        width = max,
        notifier = function(value)          
          -- Update the volume setting for the particular step
          step_data[step].volume = value
          -- Update the note button length display
          refresh_points(step_data, step)        
        end
      },   
 
 
      -- Filter controls
      vb:row {
        margin = -3,
        spacing = -8,
        
        vb:minislider {
          id = "cutoff_step"..step,
          --tooltip = "Cutoff",
          min = 0,
          max = 1,
          value = 1,
          width = 28,
          height = 60,
          notifier = function(value)
            value = curve(value, curve_shape) -- Apply a curve to the control          
            -- Update the cutoff setting for the particular step
            step_data[step].cutoff = value
            -- Update the note button length display            
            refresh_cutoff(step_data, step)  
          end
        },
        
        vb:minislider {
          id = "resonance_step"..step,
          --tooltip = "Resonance",
          min = 0,
          max = 1,
          value = 0,
          width = 18,
          height = 60,
          notifier = function(value)          
            -- Update the resonance setting for the particular step
            step_data[step].resonance = value
            -- Update the note button length display            
            refresh_resonance(step_data, step)  
          end
        }                
      }                 

    }
    -- Add the controls to the button column        
    button_column:add_child(vb:space { height = 8 })
    button_column:add_child(step_controls)
    -- Add the button column to steps row    
    sequencer_row:add_child(button_column) 
  end  


  -- Filter controls sections
  local controls_size = 80
  local filter_types = {
    "LP 2X2 Pole",
    "LP 2 Pole",
    "LP Biquad",
    "LP Moog",
    "LP 1 Pole",
    "HP 2X2 Pole",
    "HP 2 Pole",
    "HP Moog",
    "Band Reject",
    "Band Pass",
    "EQ -15 dB",
    "EQ -6 dB",
    "EQ +6 dB",
    "EQ +15 dB",    
    "EQ Peaking",
    "Dist. Low",
    "Dist. Mid",
    "Dist. High",
    "Dist",
    "Ring Mod"
  }


  -- Function to display Help & Info popup window
  local function info()

    local vb = renoise.ViewBuilder()

    local dialog_title = "Help & Info"
    local dialog_buttons = {"OK"};
    local DEFAULT_MARGIN = renoise.ViewBuilder.DEFAULT_CONTROL_MARGIN

    local dialog_content = vb:column {
      margin = DEFAULT_MARGIN,

      vb:column {
        style = "group",
        margin = DEFAULT_MARGIN,

        vb:multiline_text {
          width = 280,
          height = 30,
          font = "big",
          style = "strong",
          text = "ReSeq v"..tool_version.." by afta8"
        },

        vb:text {
          font = "big",
          text = "Usage:"
        },
        
        vb:multiline_text {         
          width = 280,
          height = 150,
          font = "mono",
          style = "body",
          paragraphs = {"Use the piano roll to enter your notes. Underneath this are a set of sliders and dials.", 
                        "-Adjust the note lengths using the horizontal slider.\n"..
                        "-Adjust volume for each step using the dials.\n"..
                        "-Adjust filter cutoff using the fat vertical slider.\n"..
                        "-Adjust filter resonance using the thin vertical slider.\n",
                        "Filter envelope follower:\n"..
                        "-Use the large dial to adust the intensity.\n".. 
                        "-Use XY Pads to set attack and release.\n".. 
                        "-Horizontal axis controls attack.\n"..
                        "-Vertical axis controls release.\n",
                        "Adjust all steps:\n"..
                        "-Increase or decrease all step values at once.\n"..
                        "-Adjust the four dials to change note length, volume, cutoff and resonance."
                       }
        },
        
        vb:space { height = 5 },
        
        vb:multiline_text {
          width = 280,
          height = 25,
          font = "big",
          style = "strong",
          text = "You have launched this tool "..options.launch_count.value.." times." 
        },
        
        vb:multiline_text { 
          width = 280,
          height = 50,        
          font = "mono",
          style = "body",
          paragraphs = {"If you like my tools please consider making a donation, your support helps further development of Renoise tools, donate via:"}              
        },
        
        vb:text {
          font = "bold",
          text = "PayPal:  fathand@gmail.com\n"..
                 "Flattr:  flattr.com/profile/afta8\n"..
                 "\n"..
                 "For my other tools visit: afta8.tumblr.com"   
        },
        
        vb:space { height = 10 },
        
        vb:text {
          font = "big",
          text = "Experimental settings" 
        },
                        
        vb:row {                    
          vb:text {
            font = "mono",
            text = "Cutoff curve: "
          },
          vb:valuebox {
            min = 1,
            max = 20,
            value = options.cutoff_curve.value*10,
            tooltip = "1 & 2 for Log curve, 0.1 & 0.2 for exponential curve",
            tostring = function(value) 
              return (tostring(tonumber(value)/10))
            end,
            tonumber = function(str) 
              return tonumber(str)*10
            end,                
            notifier = function(value) 
              curve_shape = value / 10
              options.cutoff_curve.value = curve_shape
            end            
          }
        }
      }
    }
    renoise.app():show_custom_dialog(dialog_title, dialog_content)
  end


  -- Filter envelope follower controls
  local filter_controls = vb:row {
    style = "group",
    margin = 6,
    
    vb:column {
      
      vb:row {
        vb:checkbox {
          id = "cutoff_active",
          tooltip = "Enables the envelope follower for the filter cutoff",
          value = false,
          notifier = function(value)
            if value then
              renoise.song().instruments[ins].sample_envelopes.cutoff.follower.enabled = true
              vb.views["cutoff_amount"].active = true
              vb.views["cutoff_xypad"].active = true
            else
              renoise.song().instruments[ins].sample_envelopes.cutoff.follower.enabled = false
              vb.views["cutoff_amount"].active = false
              vb.views["cutoff_xypad"].active = false
            end
          end
        },
      
        vb:text {
        font = "mono",
        text = "Cutoff",
        tooltip = "Enables the envelope follower for the filter cutoff"
        }
      },
      
      vb:rotary {
        id = "cutoff_amount",
        tooltip = "Intensity of the envelope follower (Range = -127 to 127)",
        active = false,
        min = -127,
        max = 127,
        value = 0,
        width = 65,
        height = 65,
        notifier = function(value)                
          renoise.song().instruments[ins].sample_envelopes.cutoff.follower.amount = value
        end
      }
    },
    vb:space { width = 6 },
    vb:xypad {
      id = "cutoff_xypad",
      tooltip = "Attack and Release - x = Attack, y = Release",
      active = false,
      width = controls_size,
      height = controls_size,
      min = {x=1,y=1},
      max = {x=180,y=127},
      value = {x=20, y=80},
      notifier = function(value)        
        renoise.song().instruments[ins].sample_envelopes.cutoff.follower.attack = value.x
        renoise.song().instruments[ins].sample_envelopes.cutoff.follower.release = value.y
      end
    },

    vb:space { width = 14 },
    
    vb:column {
    
     vb:row {
        vb:checkbox {
          id = "resonance_active",
          tooltip = "Enables the envelope follower for the filter resonance",
          value = false,
          notifier = function(value)
            if value then
              renoise.song().instruments[ins].sample_envelopes.resonance.follower.enabled = true
              vb.views["resonance_amount"].active = true
              vb.views["resonance_xypad"].active = true
            else
              renoise.song().instruments[ins].sample_envelopes.resonance.follower.enabled = false
              vb.views["resonance_amount"].active = false
              vb.views["resonance_xypad"].active = false           
            end
          end
        },
      
        vb:text {
        font = "mono",
        text = "Res",
        tooltip = "Enables the envelope follower for the filter resonance"
        }
      },
      
      vb:rotary {
        id = "resonance_amount",
        tooltip = "Intensity of the envelope follower (Range = -127 to 127)",
        active = false,
        min = -127,
        max = 127,
        value = 0,
        width = 65,
        height = 65,
        notifier = function(value)                
          renoise.song().instruments[ins].sample_envelopes.resonance.follower.amount = value
        end
      }
    },
    vb:space { width = 6 },
    vb:xypad {
      id = "resonance_xypad",
      tooltip = "Attack and Release - x = Attack, y = Release",
      active = false,
      width = controls_size,
      height = controls_size,
      min = {x=1,y=1},
      max = {x=180,y=127},
      value = {x=20, y=80},
      notifier = function(value)        
        renoise.song().instruments[ins].sample_envelopes.resonance.follower.attack = value.x
        renoise.song().instruments[ins].sample_envelopes.resonance.follower.release = value.y
      end
    },
    
    vb:space { width = 16 },
    
    vb:column {
    
      style = "group",
      margin = 5,
    
      -- Filter select dropdown
      vb:row {      
        vb:text {
            width = 42,
            font = "mono",
            text = "Filter"
            },        
        vb:popup {
          id = "filter_select",
          items = filter_types,
          value = 1,
          width = 85,
          notifier = function(value)
            renoise.song().instruments[ins].sample_envelopes.filter_type = value
          end  
        }
      },

      
      vb:space { height = 2 },
      
      -- Scale dials
      vb:row {
        
        -- Gate scale
        vb:rotary {
          id = "gate_scale",
          tooltip = "Note Length",
          min = -1,
          max = 1,
          value = 0,
          width = 32,
          height = 32,
          notifier = function(value)   
            local sign = sign(value - prev_gate_val) -- Work out slider direction
            prev_gate_val = value -- remember tha last value 
            -- Set the sliders
            for step = 1, num_steps do          
              local new_val = (sign*0.3) + vb.views["length_step"..step].value -- Calculate new slider value           
              vb.views["length_step"..step].value = clamp_value(new_val, min, max) -- Set the slider
            end
          end
        },
                
        -- Volume scale
        vb:rotary {
          id = "volume_scale",
          tooltip = "Volume",
          min = -1,
          max = 1,
          value = 0,
          width = 32,
          height = 32,
          notifier = function(value)   
            local sign = sign(value - prev_cutoff_val) -- Work out slider direction
            prev_cutoff_val = value -- remember tha last value 
            -- Set the sliders
            for step = 1, num_steps do          
              local new_val = (scale_sens * sign) + vb.views["volume_step"..step].value -- Calculate new slider value           
              vb.views["volume_step"..step].value = clamp_value(new_val, 0, 1) -- Set the slider
            end
          end
        },
                
        -- Cutoff scale
        vb:rotary {
          id = "cutoff_scale",
          tooltip = "Cutoff",
          min = -1,
          max = 1,
          value = 0,
          width = 32,
          height = 32,
          notifier = function(value)   
            local sign = sign(value - prev_cutoff_val) -- Work out slider direction
            prev_cutoff_val = value -- remember the last value 
            -- Set the sliders
            for step = 1, num_steps do          
              local new_val = (scale_sens * sign) + vb.views["cutoff_step"..step].value -- Calculate new slider value           
              vb.views["cutoff_step"..step].value = clamp_value(new_val, 0, 1) -- Set the slider
            end
          end
        },
            
        -- Resonance scale
        vb:rotary {
          id = "resonance_scale",
          tooltip = "Resonance",
          min = -1,
          max = 1,
          value = 0,
          width = 32,
          height = 32,
          notifier = function(value)   
            local sign = sign(value - prev_cutoff_val) -- Work out slider direction
            prev_cutoff_val = value -- remember the last value 
            -- Set the sliders
            for step = 1, num_steps do          
              local new_val = (scale_sens * sign) + vb.views["resonance_step"..step].value -- Calculate new slider value           
              vb.views["resonance_step"..step].value = clamp_value(new_val, 0, 1) -- Set the slider
            end
          end
        } 
              
      }, 

      vb:space { height = 1 },
      
      vb:row {
      
        vb:text {
          width = 110,
          font = "mono",
          text = "Adjust All Steps"
        },        

        vb:button {
          width = 17,
          height = 17,
          bitmap = "afta8_icon.bmp",
          color = {1,1,1},
          pressed = function()
            info()
          end
        }
        
      } -- Row end

    } -- Column end
  }
        
  -- The content of the dialog, built with the ViewBuilder.
  local content = vb:column {
    margin = 5,
    vb:column {
      style = "border",
      margin = 10,
      sequencer_row,
      filter_controls   
    }  
  } 

  -- Key passthrough function
  local function keyhandler(dialog, key)
    return key
  end
  
  -- A custom dialog is non-modal and displays a user designed layout built with the ViewBuilder.   
  dialog = renoise.app():show_custom_dialog(tool_name, content, keyhandler) 


  -- Once GUI loaded, load any previous data
  local reseq = false
  
  -- Check sample names to see if save marker created
  for sample = 1, #renoise.song().instruments[ins].samples do
    if renoise.song().instruments[ins].samples[sample].name == "[ReSeq woz ere]" then 
      reseq = true
    end
  end  
  
  -- Check if ReSeq has been used on this instrument
  if reseq ~= true then -- If first use then initialise instrument
    
    -- Populate step_data with initial settings
    for step = 1, num_steps do
      step_data[step] = { note = 0, length = max, volume = 1, cutoff = 1, resonance = 0 }
    end

    -- Initialise the envelope settings
    initialise_envelopes()

    -- Add initial envelope points
    for step = 1, num_steps do
      refresh_points(step_data, step)
      refresh_cutoff(step_data, step)
      refresh_resonance(step_data, step)
    end
    
    
    -- Initialise filter type
    vb.views["filter_select"].value = renoise.song().instruments[ins].sample_envelopes.filter_type
        
    --Initialise envelope follower controls
    vb.views["cutoff_active"].value = false
    renoise.song().instruments[ins].sample_envelopes.cutoff.follower.enabled = false -- Check if this is needed
    vb.views["cutoff_amount"].value = 0
    renoise.song().instruments[ins].sample_envelopes.cutoff.follower.amount = 0  -- Check if this is needed
    vb.views["cutoff_xypad"].value = {x = renoise.song().instruments[ins].sample_envelopes.cutoff.follower.attack, 
                                      y = renoise.song().instruments[ins].sample_envelopes.cutoff.follower.release}

    vb.views["resonance_active"].value = false
    renoise.song().instruments[ins].sample_envelopes.resonance.follower.enabled = false -- Check if this is needed
    vb.views["resonance_amount"].value = 0 
    renoise.song().instruments[ins].sample_envelopes.resonance.follower.amount = 0  -- Check if this is needed
    vb.views["resonance_xypad"].value = {x = renoise.song().instruments[ins].sample_envelopes.resonance.follower.attack, 
                                         y = renoise.song().instruments[ins].sample_envelopes.resonance.follower.release} 
    
    
    -- Create ReSeq marker
    local marker_sample = #renoise.song().instruments[ins].samples + 1
    renoise.song().instruments[ins]:insert_sample_at(marker_sample)
    renoise.song().instruments[ins].samples[marker_sample].sample_buffer:create_sample_data( 1, 8, 1, 1)
    renoise.song().instruments[ins].samples[marker_sample].name = "[ReSeq woz ere]"

  else -- Load data from instrument
    
    -- Read note data and update GUI
    for step = 1, num_steps do
      local step_tick = env_tick(step) -- Convert the step value to the tick value
      local step_value -- Variable for the note value
      if p_env:has_point_at(step_tick) then
        -- Find the value for that time point
        for p = 1, #p_env.points do
          if p_env.points[p].time == step_tick then
            step_value = p_env.points[p].value
          end
        end
        -- Update data table with converted value (1-25 note format)  
        step_data[step].note = round_step(((step_value*24)+1), 1)
        -- Update button display
        vb.views["button_light_step"..step.."note"..step_data[step].note].color = on_colour               
      else
        -- There is no note data at that point
        step_data[step].note = 0
        step_data[step].volume = 0
      end  
    end
            
    -- Read note lengths and update GUI
    for step = 1, num_steps do
      local step_tick = env_tick(step) -- Convert the step value to the tick value
      local length_value = env_tpl
      -- Step through next 5 ticks to check if there is a note off
      for tick = 1, 5 do
        if v_env:has_point_at(step_tick+tick) then
          length_value = tick 
        end
      end
      -- Convert length value to GUI format
      length_value = (length_value/env_tpl)*max
      -- Update GUI
      vb.views["length_step"..step].value = length_value
    end
      
    -- Read volumes and update GUI
    for step = 1, num_steps do
      local step_tick = env_tick(step) -- Convert the step value to the tick value
      local volume_value = 1
      if step_data[step].note ~= 0 then -- If there is an active note there
        if v_env:has_point_at(step_tick) then
          -- Find the value for that point
          for p = 1, #v_env.points do
            if v_env.points[p].time == step_tick then
              volume_value = v_env.points[p].value 
            end
          end
        end        
        -- Update the GUI with the volume value
        vb.views["volume_step"..step].value = volume_value
      end
    end
       
    -- Read filter cutoff and update GUI
    for step = 1, num_steps do
      local step_tick = env_tick(step) -- Convert the step value to the tick value
      local cutoff_value = 1
      if c_env:has_point_at(step_tick) then
        -- Find the value for that point
        for p = 1, #c_env.points do
          if c_env.points[p].time == step_tick then
            cutoff_value = c_env.points[p].value 
          end
        end 
        -- Scale the value back to linear control
        cutoff_value = curve(cutoff_value, curve_shape, true)       
        -- Update the GUI with the volume value
        vb.views["cutoff_step"..step].value = cutoff_value
      end
    end    
    
    -- Read resonance and update GUI
    for step = 1, num_steps do
      local step_tick = env_tick(step) -- Convert the step value to the tick value
      local res_value = 0
      if r_env:has_point_at(step_tick) then
        -- Find the value for that point
        for p = 1, #r_env.points do
          if r_env.points[p].time == step_tick then
            res_value = r_env.points[p].value 
          end
        end        
        -- Update the GUI with the volume value
        vb.views["resonance_step"..step].value = res_value
      end
    end    
  
    -- Read filter type  
    vb.views["filter_select"].value = renoise.song().instruments[ins].sample_envelopes.filter_type
  
    -- Read envelope follower controls
    vb.views["cutoff_active"].value = renoise.song().instruments[ins].sample_envelopes.cutoff.follower.enabled 
    vb.views["cutoff_amount"].value = renoise.song().instruments[ins].sample_envelopes.cutoff.follower.amount
    vb.views["cutoff_xypad"].value = {x = renoise.song().instruments[ins].sample_envelopes.cutoff.follower.attack, 
                                      y = renoise.song().instruments[ins].sample_envelopes.cutoff.follower.release}
    vb.views["resonance_active"].value = renoise.song().instruments[ins].sample_envelopes.resonance.follower.enabled
    vb.views["resonance_amount"].value = renoise.song().instruments[ins].sample_envelopes.resonance.follower.amount
    vb.views["resonance_xypad"].value = {x = renoise.song().instruments[ins].sample_envelopes.resonance.follower.attack, 
                                         y = renoise.song().instruments[ins].sample_envelopes.resonance.follower.release}          

  end

end


--------------------------------------------------------------------------------
-- Menu entries
--------------------------------------------------------------------------------

renoise.tool():add_menu_entry {
  name = "Main Menu:Tools:"..tool_name.."...",
  invoke = show_dialog  
}

renoise.tool():add_menu_entry {
  name = "Instrument Box:"..tool_name,
  invoke = show_dialog 
}

--------------------------------------------------------------------------------
-- Key Binding
--------------------------------------------------------------------------------

---[[
renoise.tool():add_keybinding {
  name = "Global:Tools:" .. tool_name.."...",
  invoke = show_dialog
}
--]]


--------------------------------------------------------------------------------
-- MIDI Mapping
--------------------------------------------------------------------------------

--[[
renoise.tool():add_midi_mapping {
  name = tool_id..":Show Dialog...",
  invoke = show_dialog
}
--]]
